// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

varying vec4 color;

#ifdef FOG
varying vec4 fogColor;
#endif

#ifdef GL_FRAGMENT_PRECISION_HIGH
	varying highp vec3 cPos;
#else
	varying mediump vec3 cPos;
#endif
varying POS3 wPos;

#include "uniformShaderConstants.h"
#include "util.h"
#include "ESBEutil.h"
uniform vec2 FOG_CONTROL;

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

float filmic_curve(float x) {
	float A = 0.20;									// Shoulder strength
	float B = 0.30;									// Linear strength
	float C = 0.15 * TM_BRIGHTNESS;	// Linear angle
	float D = 0.20 * TM_GAMMA;			// Toe strength
	float E = 0.02 * TM_CONTRAST;		// Toe numerator
	float F = 0.30;									// Toe denominator
	return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
}

vec3 ESBEmapping(vec3 clr) {
	#ifdef ESBE_TONEMAP
		float W = 1.0 / TM_EXPOSURE;
		float Luma = dot(clr, vec3(0.7, 0.5, 0.3));
		vec3 Chroma = clr - Luma;
		clr = (Chroma * TM_SATURATION) + Luma;
		clr = vec3(filmic_curve(clr.r), filmic_curve(clr.g), filmic_curve(clr.b)) / filmic_curve(W);
	#endif
	return clr;
}

float side_shading(float dusk){
	dusk = dusk*0.75+0.25;
	vec3 n = normalize(vec3(cross(dFdx(cPos),dFdy(cPos))));
	n.x = abs(n.x*mix(1.5,0.8,dusk));
	n.yz = n.yz*0.5+0.5;
	n.yz *= mix(vec2(0.5,0.0),vec2(1.0),dusk);
	return max(n.x,max(n.y,n.z));
}


void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0);
	return;
#else

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
#endif

#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
		#define ALPHA_THRESHOLD 0.05
	#else
		#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

vec4 inColor = color;

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse *= texture2D( TEXTURE_1, uv1 );
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif
	diffuse.rgb *= inColor.rgb;
#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

#ifdef FOG
	float weather = smoothstep(0.8,1.0,FOG_CONTROL.y);
#else
	float weather = 1.0;
#endif
float daylight = texture2D(TEXTURE_1,vec2(0.0, 1.0)).r;
float dusk = smoothstep(0.1,0.6,abs(smoothstep(0.4,0.8,daylight)-0.5));
daylight *= weather;

#ifdef ESBE_LIGHT
	diffuse.rgb += ESBE_LIGHT*max(0.0,uv1.x-0.5)*mix(1.0,smoothstep(1.0,0.8,uv1.y)*0.5+0.5,daylight);
#endif

#ifdef ESBE_SUN_LIGHT
diffuse.rgb += (vec3(0.9)-diffuse.rgb)*diffuse.rgb*smoothstep(0.87-blur,0.87+blur,uv1.y)*daylight*ESBE_SUN_LIGHT;
#endif


diffuse.rgb = ESBEmapping(diffuse.rgb);

#ifdef ESBE_SHADOW
	float s_amount = mix(0.0,0.45,smoothstep(0.87 + blur,0.87 - blur,uv1.y));
	float nolight = 1.0-uv1.x;
	diffuse.rgb = mix(diffuse.rgb,ESBE_SHADOW,s_amount*nolight*daylight);
#endif

#ifdef ESBE_SIDE_SHADOW
diffuse.rgb *= mix(1.0,side_shading(dusk),smoothstep(0.7,0.95,uv1.y)*min(1.25-uv1.x,1.0)*daylight);
#endif

#ifdef DUSK
	float dusk3 = DUSK*smoothstep(dusk1,dusk1+dusk2,uv1.y)*weather;
	dusk = dusk*dusk3+(1.0-dusk3);
	diffuse.rgb *= vec3(2.0-dusk,1.0,dusk);
#endif

#ifdef FOG
	diffuse.rgb = mix( diffuse.rgb, fogColor.rgb, fogColor.a );
#endif

gl_FragColor = diffuse;

#endif // BYPASS_PIXEL_SHADER
}
